let socket;
let serverStatus = document.getElementById('serverStatus');
let isAuthenticated = false;
let lastCommandSent = null;
let keysCount = 0;
let currentKey = null;
let customQuery = null;
let originalKeys = [];
let isgraphView = false;
let currentKeyData = null;
let currentPage = 1;
let presetRecords = null;
let totalRecords = 0;
let pageSize = 100;
let totalPages = 0;
let currentActiveInput = null;

const keyDetails = document.getElementById('keyDetails');
const graphView = document.getElementById('graphView');
const buttonNewQuery = document.getElementById('buttonNewQuery');
const buttonNewKey = document.getElementById('buttonNewKey');
const buttonIndices = document.getElementById('buttonIndices');
const buttonRestore = document.getElementById('buttonRestore');
const offlineView = document.getElementById('offlineView');
const homeView = document.getElementById('homeView');
const keysList = document.getElementById('keysList');
const keysCountText = document.getElementById('keysCount');
const keysContainer = document.getElementById('keysList');

function changeServerStatus(status) {
    if(status === 'active') {
        offlineView.classList.add('hide');
        homeView.classList.remove('hide');
        keysList.classList.remove('hide');
        serverStatus.classList.remove('inactive');
        serverStatus.classList.add('active');
        enableButtons();
    } else {
        offlineView.classList.remove('hide');
        homeView.classList.add('hide');
        keysList.classList.add('hide');
        keysCountText.innerHTML = '';
        serverStatus.classList.remove('active');
        serverStatus.classList.add('inactive');
        disableButtons();
    }
}

function enableButtons() {
    var buttons = document.querySelectorAll('button');
    buttons.forEach(function(button) {
        button.disabled = false;
    });

    var inputs = document.querySelectorAll('input');
    inputs.forEach(function(input) {
        input.disabled = false;
    });

    var selects = document.querySelectorAll('select');
    selects.forEach(function(select) {
        select.disabled = false;
    });
}

function disableButtons() {
    var buttons = document.querySelectorAll('button');
    buttons.forEach(function(button) {
        button.disabled = true;
    });

    var inputs = document.querySelectorAll('input');
    inputs.forEach(function(input) {
        input.disabled = true;
    });

    var selects = document.querySelectorAll('select');
    selects.forEach(function(select) {
        select.disabled = true;
    });
}

function connectToWebSocket() {
    const uri = 'ws://127.0.0.1:6446';
    if (typeof socket !== 'undefined' && socket.readyState !== WebSocket.CLOSED) {
        return; // Exit if already connected
    }

    socket = new WebSocket(uri);

    socket.onopen = function () {
        changeServerStatus('active');
        const authData = JSON.stringify({
            username: '',
            password: ''
        });
        socket.send(authData);
    };

    socket.onmessage = function (event) {
        if (!isAuthenticated) {
            // Mark authenticated and send initial KEYS command
            isAuthenticated = true;
            lastCommandSent = 'KEYS';
            socket.send(lastCommandSent);
        } else {
            // Split the command to handle primary and secondary parts
            const commandParts = lastCommandSent.split(' ');
            const primaryCommand = commandParts[0];
            const secondaryCommand = commandParts[1] || '';

            // Handle the response data based on the primary command
            switch (primaryCommand) {
                case 'KEYS':
                    handleKeysResponse(event.data);
                    break;
                case 'CONFIG':
                    if (secondaryCommand === 'SHOW') {
                        handleConfigShowResponse(event.data);
                    } else if (secondaryCommand === 'SET') {
                        lastCommandSent = 'CONFIG SHOW';
                        socket.send(lastCommandSent);
                        showToast(event.data);
                    } else {
                        // Add additional CONFIG sub-command handling as necessary
                    }
                    break;
                case 'COUNT':
                    handleCountResponse(event.data);
                    break;
                case 'INDICES':
                    if (secondaryCommand === 'LIST') {
                        showDataInTerminalBody(event.data);
                    }
                    break;
                case 'QUERY':
                    showDataInTerminalBody(event.data);
                    break;
                case 'BACKUP':
                    if (secondaryCommand === 'LIST') {
                        handleBackupListResponse(event.data);
                    } else {
                        showToast(event.data);
                    }
                    break;
                case 'SET':
                    showToast(event.data === 'OK' ? 'Operation successful!' : 'Operation failed!');
                    break;
            }
        }
    };

    socket.onclose = function () {
        changeServerStatus('inactive');
        isAuthenticated = false; // Reset authentication flag

        // Clear the interval if the WebSocket is closed
        setTimeout(connectToWebSocket, 5000);
    };

    socket.onerror = function (error) {
        console.error('WebSocket error:', error);
        changeServerStatus('inactive');
        isAuthenticated = false; // Reset authentication flag

        // Clear the interval if there's an error
        setTimeout(connectToWebSocket, 5000);
    };
}

function handleConfigShowResponse(data) {
    const configData = JSON.parse(data);
    displayConfigData(configData);
}

function displayConfigData(config) {
    const container = document.getElementById('configCards');
    let value;
    container.innerHTML = ''; // Clear previous content

    // Create cards for each configuration item
    Object.keys(config).forEach(key => {
        // Skip creating the card for specific keys
        if (key === 'INSTANCE_UUID'
        || key === 'HOST'
        || key === 'PORT'
        || key === 'USERNAME'
        || key === 'PASSWORD'
        || key === 'REPLICATION_TYPE'
        || key === 'REPLICATION_MASTER'
        || key === 'SHARDING_TYPE'
        || key === 'SHARDS'
        || key === 'REPLICATION_SLAVES') {
            return;
        } else {
            value = config[key];
        }

        const card = document.createElement('div');
        card.className = 'config-card';

        const paragraph = createEditableParagraph(key, value);

        // Set the card content
        card.innerHTML = `<h3>${key}</h3>`;
        card.appendChild(paragraph);

        // Append the card to the container
        container.appendChild(card);
    });
}

// Helper function to create an editable paragraph
function createEditableParagraph(key, value) {
    const paragraph = document.createElement('p');
    paragraph.classList.add('pointer');
    const originalValue = value;
    let isEditing = false;
    let isUpdated = false;

    // Set the paragraph's initial value
    paragraph.textContent = value;

    // Switch to editing mode on click for other keys
    paragraph.addEventListener('click', function () {
        if (isEditing) return;

        isEditing = true;
        const input = document.createElement('input');
        input.type = 'text';
        input.value = value;

        paragraph.textContent = '';
        paragraph.appendChild(input);
        input.focus();

        // Handle the completion of editing (blur or Enter key)
        const completeEditing = () => {
            if (isUpdated) return; // Prevent duplicate updates
            isUpdated = true;

            const newValue = input.value.trim();
            if (newValue !== originalValue) {
                updateConfigKeyValue(key, newValue);
            }

            paragraph.textContent = newValue || value;
            isEditing = false;
        };

        // Handle the Enter key
        input.addEventListener('keydown', function (event) {
            if (event.key === 'Enter') {
                completeEditing();
            }
        });

        // Clicking outside the input should confirm the change
        input.addEventListener('blur', completeEditing);
    });

    return paragraph;
}

// Function to handle the update command
function updateConfigKeyValue(key, value) {
    const formattedValue = Array.isArray(value) ? value.join(',') : value;

    lastCommandSent = 'CONFIG SET';
    socket.send(`CONFIG SET ${key} ${formattedValue}`);
}

function handleKeysResponse(data) {
    try {
        const keys = JSON.parse(data);
        if (Array.isArray(keys)) {
            updateKeysList(keys);
        }
    } catch (error) {
        console.error('Error parsing WebSocket message:', error);
    }
}

function handleCountResponse(data) {
    totalRecords = !presetRecords ? parseInt(data, 10) : presetRecords;
    totalPages = Math.ceil(totalRecords / pageSize);
    currentPage = 1; // Reset to first page
    fetchPageData();
}

function updateKeysList(keys) {
    keysCount = 0;
    // Store the original keys for filtering later
    originalKeys = [...keys];
    
    // Sort keys in ascending order
    keys.sort((a, b) => a.localeCompare(b));

    keysContainer.innerHTML = ''; // Clear previous keys

    keys.forEach(key => {
        keysCount++;
        const keyItem = document.createElement('div');
        keyItem.className = 'menu-item';
        keyItem.textContent = key;

        keyItem.addEventListener('click', () => {
            customQuery = null;
            // Remove the 'active' class from any previously selected item
            removeActiveLinksClass();

            // Add the 'active' class to the clicked item
            keyItem.classList.add('active');

            // Handle your existing logic here
            currentKey = key;
            lastCommandSent = `COUNT ${key}`;
            socket.send(lastCommandSent); // Send COUNT command to get total records
        });

        keysContainer.appendChild(keyItem);
    });

    keysCountText.innerHTML = keysCount != 0 ? `(${keysCount})` : ``;

    lastCommandSent = 'CONFIG SHOW';
    socket.send(lastCommandSent);
}

function indices() {
    if (typeof socket !== 'undefined' && socket.readyState === WebSocket.OPEN) {
        removeActiveLinksClass();
        buttonIndices.classList.add('active');
        totalRecords = 1;
        currentKey = 'Indices';
        lastCommandSent = 'INDICES LIST';
        socket.send(lastCommandSent);
    } else {
        console.error('WebSocket is not open or undefined.');
    }
}

function filterKeys() {
    const searchTerm = document.getElementById('search-keys').value.toLowerCase();
    const keysContainer = document.getElementById('keysList');
    
    // Clear the existing keys list before filtering
    keysContainer.innerHTML = '';

    const filteredKeys = searchTerm === ''
        ? originalKeys  // Use the original list if search input is empty
        : originalKeys.filter(key => key.toLowerCase().includes(searchTerm));

    // Re-populate the keys list based on the filtered or original keys
    filteredKeys.forEach(key => {
        const keyItem = document.createElement('div');
        keyItem.className = 'menu-item';
        keyItem.textContent = key;

        keyItem.addEventListener('click', () => {
            // Remove the 'active' class from any previously selected item
            removeActiveLinksClass();

            // Add the 'active' class to the clicked item
            keyItem.classList.add('active');

            // Handle your existing logic here
            currentKey = key;
            lastCommandSent = `COUNT ${key}`;
            socket.send(lastCommandSent); // Send COUNT command to get total records
        });

        keysContainer.appendChild(keyItem);
    });
}

function showDataInTerminalBody(data) {
    presetRecords = null;
    // Hide home view
    document.getElementById('homeView').style.display = 'none';

    // Reset the graphView and its content
    resetGraphView();

    keyDetails.style.display = 'block';

    let html = `<h2>Nested View for ${currentKey}</h2>`;
    html += `<p><button class="transparent" id="toggleHierarchyGraph">Show Hierarchy Graph</button></p>`;

    try {
        const parsedData = JSON.parse(data);
        currentKeyData = parsedData;
        if (typeof parsedData === 'object') {
            html += generateCollapsibleContent(parsedData);
        } else {
            html += `<pre>${formatScalar(parsedData, true)}</pre>`;
        }
    } catch (e) {
        html += `<pre>${formatScalar(data, true)}</pre>`;
    }

    html += generatePaginationControls();
    keyDetails.innerHTML = html;
    updatePaginationDisplay();

    const toggleHierarchyGraph = document.getElementById('toggleHierarchyGraph');
    toggleHierarchyGraph.onclick = () => toggleView(currentKeyData, keyDetails, graphView);
}

function toggleView(parsedData, terminalView, graphView) {
    isgraphView = !isgraphView;

    // Retrieve the toggle button
    const toggleButton = document.getElementById('toggleHierarchyGraph');

    if (isgraphView) {
        terminalView.style.display = 'none';
        graphView.style.display = 'block';
        graphView.innerHTML = ''; // Clear previous graph content

        // Create and add a new heading for the hierarchical graph
        const graphHeading = document.createElement('h2');
        graphHeading.textContent = `Hierarchy Graph for ${currentKey}`;

        // Append the heading and then the toggle button to the graph view
        graphView.appendChild(graphHeading);
        graphView.appendChild(toggleButton);
        toggleButton.textContent = 'Show Nested View';

        // Create the graph visualization
        renderGraphHierarchy(graphView, parsedData);
    } else {
        terminalView.style.display = 'block';
        resetGraphView();

        // Ensure no duplicates by removing the button if already exists
        if (toggleButton.parentNode === graphView) {
            graphView.removeChild(toggleButton);
        }

        // Find the title or heading element to insert the button right after it
        const headingElement = terminalView.querySelector('h2');

        // Insert the toggle button below the title
        if (headingElement) {
            headingElement.insertAdjacentElement('afterend', toggleButton);
        } else {
            // Fallback in case heading isn't found (insert at the top)
            terminalView.insertBefore(toggleButton, terminalView.firstChild);
        }

        toggleButton.textContent = 'Show Hierarchy Graph';
    }
}

function renderGraphHierarchy(element, data) {
    function transformFirstEntry(obj) {
        const firstKey = Object.keys(obj)[0];
        const firstValue = obj[firstKey];
        const firstNode = { name: firstKey };
        if (typeof firstValue === 'object' && firstValue !== null) {
            firstNode.children = transformData(firstValue);
        } else {
            firstNode.value = firstValue;
        }
        return [firstNode];
    }

    function transformData(obj) {
        const result = [];
        for (const [key, value] of Object.entries(obj)) {
            const node = { name: key };
            if (typeof value === 'object' && value !== null) {
                node.children = transformData(value);
            } else {
                node.value = value;
            }
            result.push(node);
        }
        return result;
    }

    const hierarchicalData = {
        name: "root",
        children: transformFirstEntry(data),
    };

    const root = d3.hierarchy(hierarchicalData);

    const padding = { top: 350, right: 150, left: 250 };
    const width = element.clientWidth - padding.right;
    const depthCount = root.height;
    const nodeCount = root.descendants().length;
    const idealHeight = nodeCount * 80;
    const height = Math.max(idealHeight, element.clientHeight);

    const treeLayout = d3.tree().size([height, width])
        .nodeSize([15, width / (depthCount + 1)])
        .separation((a, b) => a.parent === b.parent ? 1 : 1.5);

    treeLayout(root);

    const svg = d3.select(element).append('svg')
        .attr('width', width + padding.left)  // Adjusted width
        .attr('height', height)
        .style("background-color", "#333");

    const g = svg.append('g')
        .attr('transform', `translate(${padding.left},${padding.top})`);

    // Function to handle zoom behavior correctly
    function zoomed(event) {
        g.attr('transform', `translate(${event.transform.x + padding.left}, ${event.transform.y + padding.top}) scale(${event.transform.k})`);
    }

    svg.call(d3.zoom().on("zoom", zoomed));

    const links = g.append("g").selectAll("path")
        .data(root.links())
        .join("path")
        .attr("d", d3.linkHorizontal().x(d => d.y).y(d => d.x))
        .attr('fill', 'none')
        .attr('stroke', '#FFF')
        .attr('stroke-opacity', 0.6);

    const nodes = g.append("g").selectAll("g")
        .data(root.descendants())
        .join("g")
        .attr("transform", d => `translate(${d.y},${d.x})`);

    nodes.append("circle").attr("fill", "#fff").attr("r", 4);

    nodes.append("text")
        .attr("x", d => d.children ? -6 : 6)
        .attr("text-anchor", d => d.children ? "end" : "start")
        .attr("font-family", "sans-serif")
        .attr("font-size", 10)
        .text(d => d.data.name)
        .style("fill", "#FFF");
}

function formatScalar(value, editable = false, fullPath = '', fieldName = '') {
    if(currentKey === 'Indices') {
        return `<span class='non-editable-scalar'>${value.toString()}</span>`;
    }
    if (fieldName === 'key') {
        return `<span class='non-editable-scalar'>${value.toString()}</span>`;
    } else {
        return editable ? `<span class='editable-scalar pointer' data-editable='true' data-path='${fullPath}'>${value.toString()}</span>` : value.toString();
    }
}

function formatArray(array, editable = false, fullPath = '') {
    return array.map((item, index) => {
        const itemPath = `${fullPath}_${index}`;
        return formatScalar(item, editable, itemPath);
    }).join(', ');
}

function formatObject(obj, editable = false) {
    const collapsibleContent = Object.entries(obj).map(([key, val]) => {
        return `<div class="collapsible-item"><strong>${key}:</strong> ${formatScalar(val, editable)}</div>`;
    }).join('');
    return `<div class="collapsible-content">${collapsibleContent}</div>`;
}

function generateCollapsibleContent(data, parentPath = '') {
    let basePath = parentPath ? `${parentPath}` : currentKey;  // Initialize basePath either with parentPath or currentKey

    let html = `<div class="collapsible">`;

    // Handle the `key` field first if present
    if (data.hasOwnProperty('key')) {
        const keyPath = `${basePath}:key`;
        html += `
            <div class="collapsible-header" onclick="toggleCollapsible(this)">
                key <span class="collapsible-header-type">(Scalar)</span>
            </div>
            <div class="collapsible-content" style="display:none;">
                <span class="non-editable-scalar" data-path="${keyPath}">${data.key}</span>
            </div>
        `;
    }

    // Then handle all other fields excluding `key`
    Object.entries(data).forEach(([key, value]) => {
        // Skip `key` since it's already handled above
        if (key === 'key') return;

        const fieldKey = value.key || key;
        const currentPath = `${basePath}:${fieldKey}`;
        html += `
            <div class="collapsible-header" onclick="toggleCollapsible(this)">
                ${fieldKey} <span class="collapsible-header-type">(${typeof value === 'object' ? (Array.isArray(value) ? 'Array' : 'Object') : 'Scalar'})</span>
            </div>
            <div class="collapsible-content" style="display:none;">
                ${typeof value === 'object' ? generateCollapsibleContent(value, currentPath) : formatScalar(value, true, currentPath, key)}
            </div>
        `;
    });

    html += `</div>`;
    return html;
}

function makeElementEditable(element) {
    let initialValue = element.textContent;
    let input = document.createElement('input');
    input.type = 'text';
    input.value = initialValue;
    input.className = 'edit-input';
    element.innerHTML = '';
    element.appendChild(input);
    input.focus();
    element.classList.add('edit-active');

    input.addEventListener('blur', function () {
        finishEditing(element, initialValue, input.value);
    });

    input.addEventListener('keydown', function (e) {
        if (e.key === 'Enter') {
            input.blur();
            e.preventDefault(); // Prevents the addition of a new line in contenteditable
        }
    });
}

// Update the element with new value and send the corresponding command
function finishEditing(element, initialValue, newValue) {
    let fullPath = element.getAttribute('data-path');
    let command = '';
    lastCommandSent = 'SET';

    // Extract the base path and index for array elements
    const indexPattern = /(.+):(\d+)$/; // Pattern to extract the base path and index
    const match = fullPath.match(indexPattern);

    if (match) {
        const basePath = match[1];
        const index = parseInt(match[2], 10);

        // Select all elements that share the same base path to reconstruct the full array
        const allElements = document.querySelectorAll(`[data-path^="${basePath}:"]`);
        let fullArray = Array.from(allElements).map(el => el.textContent);

        // Replace the element at the specific index with the new value
        fullArray[index] = newValue;

        // Construct the SET command with the full array
        command = `SET ${basePath} [${fullArray.join(',')}]`;
    } else {
        // For scalar values, directly set the updated value
        command = `SET ${fullPath} ${newValue}`;
    }

    // Send the command only if the value has changed
    if (command && initialValue !== newValue) {
        socket.send(command);

        // Update the element with the new value
        element.textContent = newValue;
    } else {
        // Revert to the original value if no change
        element.textContent = initialValue;
    }
    element.classList.remove('edit-active'); // Remove edit mode regardless of change
}

function generatePaginationControls() {
    return `
        <div id="pagination-controls" class="pagination">
            <button onclick="navigate('first')">First</button>
            <button onclick="navigate('prev')">Previous</button>
            <span id="page-info" onclick="pageEditable(this)">Page ${currentPage} of ${totalPages}</span>
            <button onclick="navigate('next')">Next</button>
            <button onclick="navigate('last')">Last</button>
        </div>
    `;
}

function pageEditable(element) {
    // Extract the current page and total pages from the element's text
    const pageInfo = element.textContent.trim().split(' '); // ["Page", "current", "of", "total"]
    const current = parseInt(pageInfo[1], 10); // Convert current page number from string to integer
    const total = parseInt(pageInfo[3], 10); // Convert total pages number from string to integer

    const input = document.createElement('input');
    input.type = 'number';
    input.value = current;
    input.min = 1;
    input.max = total;
    input.style.width = '50px';

    // Update the text content of the element to include only 'Page' and the total pages
    element.textContent = 'Page ';
    element.appendChild(input);
    element.append(` of ${total}`);

    input.focus();

    input.onblur = function() {
        const newPage = parseInt(input.value, 10);
        if (!isNaN(newPage) && newPage >= 1 && newPage <= total) { // Use 'total' from earlier extraction
            currentPage = newPage;
            navigate('goToPage', newPage);
        } else {
            // Reset the text to show the current page if the input is invalid
            element.textContent = `Page ${currentPage} of ${total}`;
        }
    };

    input.onkeypress = function(event) {
        if (event.key === "Enter") {
            input.blur();
        }
    };
}

function toggleCollapsible(header) {
    const content = header.nextElementSibling;
    content.style.display = content.style.display === "none" ? "block" : "none";
    header.classList.toggle('active');
}

function setupEventDelegation() {
    document.querySelector('.terminal-body').addEventListener('click', function(event) {
        let targetElement = event.target;
        if (targetElement.classList.contains('editable-scalar') && !targetElement.classList.contains('edit-active')) {
            makeElementEditable(targetElement);
        }
    });
}

// Attach editable functionality to all relevant elements
document.querySelectorAll('.editable-cell, .editable-scalar').forEach(element => {
    element.addEventListener('click', function (event) {
        if (!element.isContentEditable && !element.classList.contains('edit-active')) {
            makeElementEditable(element);
        }
    });
});

function navigate(actionOrPageNumber) {
    if (typeof actionOrPageNumber === 'string') {
        switch (actionOrPageNumber) {
            case 'first':
                currentPage = 1;
                break;
            case 'prev':
                if (currentPage > 1) currentPage--;
                break;
            case 'next':
                if (currentPage < totalPages) currentPage++;
                break;
            case 'last':
                currentPage = totalPages;
                break;
        }
    } else if (typeof actionOrPageNumber === 'number') {
        // Handle numerical navigation to a specific page number
        const pageNumber = actionOrPageNumber;
        if (pageNumber >= 1 && pageNumber <= totalPages) {
            currentPage = pageNumber;
        } else {
            console.warn(`Invalid page number: ${pageNumber}. Please choose a page between 1 and ${totalPages}.`);
            return;
        }
    }

    fetchPageData();
}

function fetchPageData() {
    let offset = (currentPage - 1) * pageSize;
    let recordsToFetch = pageSize;

    // Calculate the remaining records for the last page
    if (currentPage === totalPages) {
        recordsToFetch = totalRecords - offset;
    }

    let baseQuery = `QUERY ${currentKey}`;
    if (customQuery) {
        // Regular expression to find and remove any existing LIMIT clause
        const limitRegex = /\sLIMIT\(\d+\s*(,\s*\d+)?\)/i;
        baseQuery = customQuery.replace(limitRegex, ''); // Remove existing LIMIT clause if present
    }

    // Append the correct LIMIT clause based on the current page and remaining records
    lastCommandSent = `${baseQuery} LIMIT(${offset},${recordsToFetch})`;

    socket.send(lastCommandSent);
}

function updatePaginationDisplay() {
    // Assume you have a way to calculate totalRecords or it's returned by the server
    totalPages = Math.ceil(totalRecords / pageSize);
    document.getElementById('page-info').textContent = `Page ${currentPage} of ${totalPages}`;

    document.querySelector('button[onclick="navigate(\'first\')"]').disabled = currentPage <= 1;
    document.querySelector('button[onclick="navigate(\'prev\')"]').disabled = currentPage <= 1;
    document.querySelector('button[onclick="navigate(\'next\')"]').disabled = currentPage >= totalPages;
    document.querySelector('button[onclick="navigate(\'last\')"]').disabled = currentPage >= totalPages;
}

function backup() {
    if (socket && socket.readyState === WebSocket.OPEN && confirm('Do you want to initiate a backup?')) {
        lastCommandSent = 'BACKUP';
        socket.send('BACKUP');
        restore();
        showToast("Backup complete");
    } else {
        console.error('WebSocket connection is not open.');
    }
}

function restore() {
    removeActiveLinksClass();
    buttonRestore.classList.add('active');
    resetGraphView();

    if (socket && socket.readyState === WebSocket.OPEN) {
        lastCommandSent = 'BACKUP LIST';
        socket.send('BACKUP LIST');
    } else {
        console.error('WebSocket connection is not open.');
    }
}

function handleBackupListResponse(data) {
    // Hide the home view and prepare the keyDetails area for new content
    document.getElementById('homeView').style.display = 'none';
    let keyDetails = document.getElementById('keyDetails');
    keyDetails.style.display = 'block';

    const parsedData = JSON.parse(data);
    let html = `<h2>Backup Files</h2>`;
    if (parsedData.length && !parsedData[0].message) {
        const groupedBackups = groupBackupsByDate(parsedData);
        Object.entries(groupedBackups).forEach(([date, files]) => {
            html += `<h3>${date}</h3><ul>`;
            files.forEach(file => {
                html += `<li>
                ${file.file} <button class="transparent" onclick="restoreBackup('${file.file}')">restore backup</button>
                <button class="transparent" onclick="deleteBackup('${file.file}')">delete backup</button>
                </li>`;
            });
            html += `</ul>`;
        });
    } else {
        html += `<p>${parsedData[0].message || 'Error retrieving backup files.'}</p>`;
    }
    keyDetails.innerHTML = html; // Set the content in the keyDetails area
}

function groupBackupsByDate(backups) {
    return backups.reduce((acc, backup) => {
        const date = backup.date.split(' ')[0]; // Extracting only the date part
        if (!acc[date]) {
            acc[date] = [];
        }
        acc[date].push(backup);
        return acc;
    }, {});
}

function restoreBackup(fileName) {
    if (socket && socket.readyState === WebSocket.OPEN && confirm('Do you want to restore this backup?')) {
        removeActiveLinksClass();
        lastCommandSent = 'BACKUP RESTORE';
        socket.send(`BACKUP RESTORE ${fileName}`);
        setTimeout(function() {
            refreshKeys();
        },1000);
    } else {
        console.error('WebSocket connection is not open.');
    }
}

function deleteBackup(fileName) {
    if (socket && socket.readyState === WebSocket.OPEN && confirm('Do you want to delete this backup?')) {
        lastCommandSent = 'BACKUP DELETE';
        socket.send(`BACKUP DELETE ${fileName}`);
        restore();
    } else {
        console.error('WebSocket connection is not open.');
    }
}

function createNewKeyForm() {
    removeActiveLinksClass();
    buttonNewKey.classList.add('active');
    resetGraphView();

    // Hide home view and prepare keyDetails for new content
    document.getElementById('homeView').style.display = 'none';
    let keyDetails = document.getElementById('keyDetails');
    keyDetails.style.display = 'block';

    const keys = Array.from(document.querySelectorAll('#keysList .menu-item')).map(item => item.textContent);
    let html = `<h2>Create New Key</h2>`;
    html += `<select id="key-base" class="base-key" onchange="updateFullKey()" aria-label="Select Base Key">`;
    html += `<option value="">New Key</option>`;
    keys.forEach(key => {
        html += `<option value="${key}">${key}</option>`;
    });
    html += `</select>`;
    html += `<div id="new-key-name-container" class="new-key-name-container" style="display:none;">
                <input type="text" id="new-key-name" placeholder="Key Name" oninput="updateFullKey()">
             </div>`;
    html += `<div id="key-parts" class="key-parts"></div>`;
    html += `<button class="button-small add-button" onclick="addKeyPart()">+ Add Sub Key</button>`;
    html += `<input class="new-key-value" type="text" id="key-value" placeholder="Enter Value" oninput="updateFullKey()">`;
    html += `<hr>`;
    html += `<p class="new-key-full-key">Full Key: <span id="full-key-display"></span></p>`;
    html += `<button class="transparent" onclick="submitNewKey()">Save Key</button>`;
    keyDetails.innerHTML = html;
    updateFullKey();
}

// Function to add a new subkey input field to the form
function addKeyPart() {
    const container = document.getElementById('key-parts');
    const partContainer = document.createElement('div');
    partContainer.className = 'key-part-container';

    // Create the input for the subkey
    const newPart = document.createElement('input');
    newPart.type = 'text';
    newPart.placeholder = 'Enter sub-key';
    newPart.oninput = updateFullKey;
    newPart.className = 'key-part';

    // Create the remove button
    const removeButton = document.createElement('button');
    removeButton.textContent = '-';
    removeButton.className = 'button-small remove-button';
    removeButton.onclick = function () {
        container.removeChild(partContainer);
        updateFullKey();
    };

    // Add input and remove button to the container
    partContainer.appendChild(newPart);
    partContainer.appendChild(removeButton);

    // Append the subkey container to the main parts container
    container.appendChild(partContainer);

    // Update the full key display
    updateFullKey();
}

function updateFullKey() {
    // Get the base key selection and value
    const baseKeySelect = document.getElementById('key-base');
    const baseKey = baseKeySelect.value;
    const newKeyContainer = document.getElementById('new-key-name-container');
    const keyValue = document.getElementById('key-value').value; // Get the value of the key
    let fullKey = '';

    // Check if the "New Key" option is selected
    if (!baseKey) {
        // Show new key input field and get the new key name
        newKeyContainer.style.display = 'block';
        const newKeyName = document.getElementById('new-key-name').value.trim();
        fullKey = newKeyName ? newKeyName : 'New Key';
    } else {
        // Hide new key input field and use the selected base key
        newKeyContainer.style.display = 'none';
        fullKey = baseKey;
    }

    // Collect and concatenate subkeys
    const keyParts = Array.from(document.getElementsByClassName('key-part')).map(input => input.value.trim());
    if (keyParts.length > 0 && keyParts.some(part => part !== '')) {
        fullKey += ':' + keyParts.filter(part => part !== '').join(':');
    }

    // Append the key value at the end of the full key display
    fullKey = `${fullKey} ${keyValue}`; // Append the value to the full key

    // Update the displayed full key
    document.getElementById('full-key-display').textContent = fullKey;
}

function submitNewKey() {
    const fullKey = document.getElementById('full-key-display').textContent;
    const value = document.getElementById('key-value').value;

    // Check if the WebSocket is open and required fields are provided
    if (value && fullKey && socket && socket.readyState === WebSocket.OPEN) {
        lastCommandSent = 'SET';
        socket.send(`SET ${fullKey}`);

        // Reset the form fields after successful submission
        document.getElementById('key-base').selectedIndex = 0;
        document.getElementById('key-parts').innerHTML = '';
        document.getElementById('key-value').value = '';
        updateFullKey();
    } else if (!value) {
        console.error('Please provide a value.');
    } else if (!fullKey) {
        console.error('Please provide a key base and parts.');
    } else {
        console.error('WebSocket connection is not open.');
    }
}

function createNewQueryForm() {
    removeActiveLinksClass();
    document.getElementById('buttonNewQuery').classList.add('active');
    resetGraphView();

    const queryDetails = document.getElementById('keyDetails');
    queryDetails.style.display = 'block';
    document.getElementById('homeView').style.display = 'none';

    const keys = Array.from(document.querySelectorAll('#keysList .menu-item')).map(item => item.textContent);
    let html = `<h2>Create New Query</h2>`;
    html += `<select id="query-base" class="base-key" onchange="updateQueryPreview()">`;
    html += `<option value="">Select Key</option>`;
    keys.forEach(key => {
        html += `<option value="${key}">${key}</option>`;
    });
    html += `</select>`;
    html += `<div id="sub-keys" class="sub-keys"></div>`;
    html += `<button class="button-small add-button" onclick="addQuerySubKey()">+ Add Sub Key</button>`;
    html += `<div id="query-conditions" class="query-conditions"></div>`;
    html += `<button class="button-small add-button" onclick="addQueryCondition()">+ Add Condition</button>`;
    html += generateQueryInputs();
    html += `<hr>`;
    html += `<p id="query-preview" class="new-query-preview"></p>`;
    html += `<button class="transparent" onclick="submitNewQuery()">Execute Query</button>`;
    queryDetails.innerHTML = html;
}

function addQuerySubKey() {
    const subKeysContainer = document.getElementById('sub-keys');

    const subKeyDiv = document.createElement('div');
    subKeyDiv.className = 'sub-key';

    // Input field for sub-key with oninput event
    const input = document.createElement('input');
    input.className = 'sub-key-input query-key-value';
    input.placeholder = 'Enter sub-key';
    input.oninput = updateQueryPreview;

    // Button to remove the sub-key input
    const removeButton = document.createElement('button');
    removeButton.className = 'button-small remove-button';
    removeButton.textContent = '-';
    removeButton.onclick = function () {
        subKeysContainer.removeChild(subKeyDiv);
        updateQueryPreview();
    };

    subKeyDiv.appendChild(input);
    subKeyDiv.appendChild(removeButton);

    subKeysContainer.appendChild(subKeyDiv);
}

function addQueryCondition() {
    const conditionsContainer = document.getElementById('query-conditions');

    const conditionDiv = document.createElement('div');
    conditionDiv.className = 'condition';

    // Logical operator selection (only shown if not the first condition)
    const logicalSelect = document.createElement('select');
    logicalSelect.className = 'condition-logic query-key-value';
    ['AND', 'OR'].forEach(op => {
        const option = document.createElement('option');
        option.value = op;
        option.textContent = op;
        logicalSelect.appendChild(option);
    });
    logicalSelect.oninput = updateQueryPreview;
    
    // Only add logical operator selection if not the first condition
    if (conditionsContainer.children.length > 0) {
        conditionDiv.appendChild(logicalSelect);
    }

    // Input for the field name (potentially a subkey)
    const fieldInput = document.createElement('input');
    fieldInput.type = 'text';
    fieldInput.classList.add('condition-field', 'query-key-value', 'subkey-input');
    fieldInput.placeholder = 'Field Name';
    fieldInput.oninput = updateQueryPreview;

    // Select for the comparison operation
    const operationSelect = document.createElement('select');
    operationSelect.className = 'condition-operation query-key-value';
    const operations = ['=', '>', '<', '>=', '<=', '!=', 'LIKE', 'BETWEEN'];
    operations.forEach(op => {
        const option = document.createElement('option');
        option.value = op;
        option.textContent = op;
        operationSelect.appendChild(option);
    });
    operationSelect.oninput = updateQueryPreview;

    // Input for the value being compared against
    const valueInput = document.createElement('input');
    valueInput.type = 'text';
    valueInput.className = 'condition-value query-key-value';
    valueInput.placeholder = 'Value';
    valueInput.oninput = updateQueryPreview;

    // Add a delete button to remove the condition
    const removeButton = document.createElement('button');
    removeButton.className = 'button-small remove-button';
    removeButton.textContent = '-';
    removeButton.onclick = function () {
        conditionsContainer.removeChild(conditionDiv);
        updateQueryPreview();
    };

    // Append all inputs and the remove button to the condition div
    conditionDiv.appendChild(fieldInput);
    conditionDiv.appendChild(operationSelect);
    conditionDiv.appendChild(valueInput);
    conditionDiv.appendChild(removeButton);

    // Append the entire condition div to the conditions container
    conditionsContainer.appendChild(conditionDiv);

    // Update the query preview immediately
    updateQueryPreview();
}

function generateQueryInputs() {
    return `
        <input type="text" class="query-key-value" id="limit" placeholder="LIMIT(optional)" oninput="updateQueryPreview()">
        <input type="text" class="query-key-value" id="exclude" placeholder="EXCLUDE(key1,key2,key3...)" oninput="updateQueryPreview()">
        <input type="text" class="query-key-value" id="groupby" placeholder="GROUPBY(key)" oninput="updateQueryPreview()">
        <input type="text" class="query-key-value" id="orderby" placeholder="ORDERBY(key,ASC/DESC)" oninput="updateQueryPreview()">
        <input type="text" class="query-key-value" id="join" placeholder="JOIN(key_to_join,relational_key)" oninput="updateQueryPreview()">
    `;
}

function updateQueryPreview() {
    const base = document.getElementById('query-base').value;
    const subKeys = Array.from(document.querySelectorAll('.sub-key-input')).map(input => input.value).filter(val => val.trim()).join(':');
    const conditions = Array.from(document.querySelectorAll('.condition')).map((cond, index) => {
        const logic = index > 0 ? ` ${cond.querySelector('.condition-logic').value} ` : '';
        const field = cond.querySelector('.condition-field').value;
        const operation = cond.querySelector('.condition-operation').value;
        const value = cond.querySelector('.condition-value').value;
        return `${logic}${field} ${operation} ${value}`;
    }).join('');

    let query = `QUERY ${base}${subKeys ? ':' + subKeys : ''}`;
    if (conditions) query += ` WHERE ${conditions}`;

    const limit = document.getElementById('limit').value;
    const exclude = document.getElementById('exclude').value;
    const groupby = document.getElementById('groupby').value;
    const orderby = document.getElementById('orderby').value;
    const join = document.getElementById('join').value;
    if (limit) query += ` LIMIT(${limit})`;
    if (exclude) query += ` EXCLUDE(${exclude})`;
    if (groupby) query += ` GROUPBY(${groupby})`;
    if (orderby) query += ` ORDERBY(${orderby})`;
    if (join) query += ` JOIN(${join})`;

    document.getElementById('query-preview').textContent = query;
}

function submitNewQuery() {
    removeActiveLinksClass();
    const queryPreviewElement = document.getElementById('query-preview');
    let queryPreview = queryPreviewElement.textContent.trim();

    // Extract the key and any subkeys as the currentKey
    const keyRegex = /^QUERY\s+([^\s]+)/;
    const keyMatch = queryPreview.match(keyRegex);
    currentKey = keyMatch ? keyMatch[1] : null;  // Default if no match

    if(currentKey) {
        // Extract the LIMIT values and adjust if needed
        const limitRegex = /LIMIT\((\d+)(,\s*\d+)?\)/;
        const limitMatch = queryPreview.match(limitRegex);
        presetRecords = null; // Default to null if no LIMIT is found

        if (limitMatch) {
            const limits = limitMatch[1].split(',').map(Number);
            presetRecords = limits.length === 2 ? limits[1] - limits[0] + 1 : limits[0];
            // Adjust queryPreview if necessary, to ensure consistent LIMIT clause
        } else {
            queryPreview += ` LIMIT(${pageSize})`; // Append default LIMIT
            totalRecords = pageSize;
        }

        customQuery = queryPreview;
        lastCommandSent = `COUNT ${currentKey}`;
        socket.send(lastCommandSent); // Send COUNT command to get total records
    }
}

function resetToHomeView() {
    // Display the home view
    document.getElementById('homeView').style.display = 'block';
}

function refreshKeys() {
    // Clear out the key details section
    resetGraphView();
    removeActiveLinksClass();
    resetKeyDetails();
    resetToHomeView();
    lastCommandSent = 'KEYS';
    socket.send(lastCommandSent);
}

function resetKeyDetails() {
    keyDetails.style.display = 'none';
    keyDetails.innerHTML = '';
}

function resetGraphView() {
    isgraphView = false;
    graphView.style.display = 'none';
    graphView.innerHTML = '';
}

function removeActiveLinksClass() {
    const previousActive = document.querySelector('.menu-item.active');

    if (previousActive) {
        previousActive.classList.remove('active');
    }

    if(buttonNewKey) {
        buttonNewKey.classList.remove('active');
    }

    if(buttonNewQuery) {
        buttonNewQuery.classList.remove('active');
    }

    if (buttonIndices) {
        buttonIndices.classList.remove('active');
    }

    if (buttonRestore) {
        buttonRestore.classList.remove('active');
    }
}

function showToast(message) {
    const toast = document.createElement('div');
    toast.className = 'toast';
    toast.textContent = message;
    document.body.appendChild(toast);

    setTimeout(() => {
        toast.classList.add('show');
    }, 100);

    setTimeout(() => {
        toast.classList.remove('show');
        setTimeout(() => {
            document.body.removeChild(toast);
        }, 500);
    }, 3000);
}

window.onload = function() {
    connectToWebSocket();
    setupEventDelegation();
};

document.addEventListener('DOMContentLoaded', function() {
    const leftMenu = document.getElementById('left-menu');
    const resizer = document.getElementById('resizer');
    let startX, startWidth;

    resizer.addEventListener('mousedown', function(e) {
        startX = e.clientX;
        startWidth = parseInt(window.getComputedStyle(leftMenu).width, 10);
        document.documentElement.addEventListener('mousemove', handleMouseMove, false);
        document.documentElement.addEventListener('mouseup', stopResize, false);
        e.preventDefault();
    });

    function handleMouseMove(e) {
        if (!startWidth) return;  // Prevents resizing if startWidth is not set
        let newWidth = startWidth + (e.clientX - startX);
        leftMenu.style.width = newWidth + 'px';
    }

    function stopResize() {
        document.documentElement.removeEventListener('mousemove', handleMouseMove, false);
        document.documentElement.removeEventListener('mouseup', stopResize, false);
    }
});

document.getElementById('search-keys').addEventListener('input', filterKeys);